clc  % Clear Command Window
clear %Remove items from workspace, freeing up system memory
close all % Remove specified figure

ts=0.1; % sample time
t=0:ts:60; % vector time

%%%%%%%%%%%%%%%%%%% Initial conditions %%%%%%%%%%%%%%%%%%%%%

x1(1) = 0;  % initial center position (x axis) in meters (m)
y1(1) = 0;  % initial center position (y axis) in meters (m)
phi(1)=0; % initial orientation in radians

q1(1)=0; % initial angular position q2 in radians
q2(1)=0; % initial angular position q3 in radians

a=0.2; % distance to interest point in meters (m)
b=0.25; % height of mobile robot
c=0.25; % height of  base of arm robot
l1=0.5; % length of first joint
l2=0.55;% length of second joint

% Geometric model

% mobile platform
xp(1)=x1(1)+a*cos(phi(1));
yp(1)=y1(1)+a*sin(phi(1));

% arm 
hx(1)=xp(1)+l1*cos(q1(1))*cos(phi(1))+l2*cos(q1(1)+q2(1))*cos(phi(1));   % initial interest position (x axis) in meters (m)
hy(1)=yp(1)+l1*cos(q1(1))*sin(phi(1))+l2*cos(q1(1)+q2(1))*sin(phi(1));   % initial interest position (y axis) in meters (m)
hz(1)=b+c+l1*sin(q1(1))+l2*sin(q1(1)+q2(1)); % initial interest position (z axis) in meters (m)


%%%%%%%%%%%%%%%%%%  Mobile platform velocities and arm velocities %%%%%%%%%%%%%%%%%%%%

u = 0.1*ones(1,size(t,2));  %linear velocity in m/s
w = 0.1*ones(1,size(t,2));  %  angular velocity in rad/s
q1p = 0.1*sin(0.3*t); % angular velocity in rad/s
q2p = 0.1*cos(0.2*t);  % angular velocity in rad/s



%%%%%%%%%%%%%%%%%%% Control Loop %%%%%%%%%%%%%%%%%%%%%%

for k=1:length(t)  % Loop simulation

 
%%%%%%%%%%%%%%%%%%% Kinematic Model %%%%%%%%%%%%%%%%%%%%%%%%%

        % Jacobian Matrix
        J11 = cos(phi(k));
        J12 = -a*sin(phi(k))-l1*cos(q1(k))*sin(phi(k))-l2*cos(q1(k)+q2(k))*sin(phi(k));
        J13 = -l1*sin(q1(k))*cos(phi(k))-l2*sin(q1(k)+q2(k))*cos(phi(k));
        J14 = -l2*sin(q1(k)+q2(k))*cos(phi(k));
                
        J21 = sin(phi(k));
        J22 = a*cos(phi(k))+l1*cos(q1(k))*cos(phi(k))+l2*cos(q1(k)+q2(k))*cos(phi(k));
        J23 = -l1*sin(q1(k))*sin(phi(k))-l2*sin(q1(k)+q2(k))*sin(phi(k));
        J24 = -l2*sin(q1(k)+q2(k))*sin(phi(k));
        
        J31 = 0;
        J32 = 0;
        J33 = l1*cos(q1(k))+l2*cos(q1(k)+q2(k));
        J34 = l2*cos(q1(k)+q2(k));
        
        

        J=[J11 J12 J13 J14;...
           J21 J22 J23 J24;...
           J31 J32 J33 J34];

        v = [u(k) w(k) q1p(k) q2p(k)]'; % vector of actions control
        
        hp = J*v;
   
        phi(k+1)=phi(k)+ts*w(k);   % real orientation in radians
        x1p(k)=u(k)*cos(phi(k+1)); % real velocity in meters/seconds
        y1p(k)=u(k)*sin(phi(k+1)); % real velocity in meters/seconds
        

        % numerical integral (Euler Method)
        x1(k+1)=x1(k)+ts*x1p(k);  % real center position (x axis) in meters (m)
        y1(k+1)=y1(k)+ts*y1p(k);  % real center position (y axis) in meters (m)
        
        q1(k+1)=q1(k)+ts*q1p(k);  % angular position q1 in radians (rad)
        q2(k+1)=q2(k)+ts*q2p(k);  % angular position q3 in radians (rad)       

        % c) Geometric model
        
        % mobile platform
        xp(k+1)=x1(k+1)+a*cos(phi(k+1));
        yp(k+1)=y1(k+1)+a*sin(phi(k+1));

        % arm
        hx(k+1)=hx(k)+ts*hp(1); % real position (x axis) in meters (m)
        hy(k+1)=hy(k)+ts*hp(2); % real position (y axis) in meters (m)
        hz(k+1)=hz(k)+ts*hp(3); % real position (z axis) in meters (m)m)
end

%%
%%%%%%%%%%%%%%%%%%%%% Stroboscopic movement of the ROBOT %%%%%%%%%%%%%%%%%%%
scene=figure;  % new figure
tam=get(0,'ScreenSize');
set(scene,'position',[tam(1) tam(2) tam(3) tam(4)]); % position and size figure in the screen
axis equal; % Set axis aspect ratios
axis([-5 5 -5 5 -1 2]); % Set axis limits 
% view([135 35]); % orientation figure
grid on; % Display axes grid lines
xlabel('x(m)'); ylabel('y(m)'); zlabel('z(m)'); % Label axis
camlight right % scene light

MobileRobot; % Parameters of platform robot 
M1=MobilePlot(x1(1),y1(1),phi(1));hold on % Plot robot in initial position x1,y1 and phi orientation
M2=plot3(hx(1),hy(1),hz(1),'gO','LineWidth',2); % Initial point.
M3=plot3(hx(1),hy(1),hz(1),'b','LineWidth',2); % Initial trajectory.
Arm_Parameters(1); % Parameters of arm robot
M5=Arm_Plot3D(xp(1),yp(1),b,0,0,phi(1),q1(1),q2(1),0);% Plot robot in initial position hx,hy and phi orientation

step=10; % step for simulation

for i=1:step:length(t) % Loop emulation
     delete (M1) % delete trajectory
     delete (M3) % delete platform robot
     delete (M5) % delete arm robot
     
     M1=MobilePlot(x1(i),y1(i),phi(i)); hold on; % Plot platform robot again
     M3=plot3(hx(1:i),hy(1:i),hz(1:i),'b','LineWidth',2); % plot trajectory.
     M5=Arm_Plot3D(xp(i),yp(i),b,0,0,phi(i),q1(i),q2(i),0);% Plot arm robot again
    pause(ts) % sample time
end

%%

scene1=figure;
set(scene1,'position',[tam(1) tam(2) tam(3) tam(4)]); % position and size figure in the screen
subplot(211)
plot(t,u,'r','LineWidth',2), grid on, xlabel('Time (s)'); ylabel('linear velocity (m/s)'); % Label axis
subplot(212)
plot(t,w,'b','LineWidth',2), grid on, xlabel('Time (s)'); ylabel('angular velocity (rad/s)');


